/*
 * Framebuffer Fill Test.
 *  
 * Copyright (C) 2005-2010 Ingenic Semiconductor Inc.
 * Author: River Wang <zwang@ingenic.cn>
 *		
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <string.h>

#define FB_W	480
#define FB_H	272
#define BPP	32

int main(int argc, char **argv)
{
	void *fb, *mem;
	
	uint32_t size = FB_W * FB_H * BPP / 8;
	uint32_t bpp_byte = BPP / 8;
	uint32_t word;

	char *p;

	int fd;	

	uint32_t i;
	int rv;

	if (argc != 3)
		return -1;
	
	word = strtoul(argv[2], &p, 16);	
	if (*p != '\0') {
		fprintf(stderr, "Invalid value.\n");
		exit(EXIT_FAILURE);
	}
	
	fprintf(stderr, "Word: 0x%x, Bpp_byte: 0x%u.\n", word, bpp_byte);

	fd = open(argv[1], O_RDWR);
	if (fd == -1) {
		perror("open():");
		exit(EXIT_FAILURE);
	}
	
	mem = fb = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0L);
	if (fb == (void *)-1) {
		perror("mmap():");
		exit(EXIT_FAILURE);
	}
	
	for (i = 0; i < size; i+= bpp_byte) {
		memcpy(fb, &word, bpp_byte);
		fb += bpp_byte;
	}
	
	munmap(mem, size);

	close(fd);

	return 0;
}
